/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************************
 * Includes
 ******************************************************************************/

#include "fsl_debug_console.h"
#include "board.h"
#include "fsl_rtc.h"
#include "fsl_power.h"
#include "power_mode_switch.h"
#include "clock_config.h"
#include "fsl_iocon.h"

#include "pin_mux.h"
#include "fsl_calibration.h"
#include "fsl_usart.h"
/* #include "nvds.h"
#include "fsl_syscon.h" */
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define DEMO_USART USART0
#define DEMO_USART_BAUDRATE 115200
#define DEMO_USART_CLK_FREQ CLOCK_GetFreq(kCLOCK_BusClk)
#define RTC_INTERRUPT_THRESHOLD (32768U * 10U)

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
extern void BOARD_WakeupRestore(void);

/*******************************************************************************
 * Variables
 ******************************************************************************/
volatile uint8_t g_RtcFreeRunningFlag;

/*******************************************************************************
 * Code
 ******************************************************************************/
/*flash_config_t g_flash_cfg;*/

/* Reinitialize peripherals after waked up from PD, this function will be called in critical area */
void BOARD_WakeupRestore(void)
{
    //    BUTTON_EnableInterrupt();

    /*
     * config.baudRate_Bps = 115200U;
     * config.parityMode = kUART_ParityDisabled;
     * config.stopBitCount = kUART_OneStopBit;
     * config.loopback = false;
     * config.enableTx = false;
     * config.enableRx = false;
     */
    usart_config_t config;
    USART_GetDefaultConfig(&config);
    config.baudRate_Bps = DEMO_USART_BAUDRATE;
    config.enableTx = true;
    config.enableRx = true;
    USART_Init(DEMO_USART, &config, DEMO_USART_CLK_FREQ);
}


void delay(uint32_t dly)
{
    volatile uint32_t i;

    for (i = 0U; i < dly; ++i)
    {
        __asm("NOP");
    }
}

void RTC_FR_IRQHandler(void)
{
    //    if (RTC_GetStatusFlags(RTC) & (uint32_t)kRTC_FreeRunningInterruptFlag)
    {
        g_RtcFreeRunningFlag = 1U;
    }
}

static void switch_to_OSC32M(void)
{
    POWER_WritePmuCtrl1(SYSCON, SYSCON_PMU_CTRL1_OSC32M_DIS_MASK, SYSCON_PMU_CTRL1_OSC32M_DIS(0U));
    SYSCON->CLK_CTRL = (SYSCON->CLK_CTRL & ~SYSCON_CLK_CTRL_SYS_CLK_SEL_MASK) | SYSCON_CLK_CTRL_SYS_CLK_SEL(0U);
}

static void switch_to_XTAL(void)
{
    /* switch to XTAL after it is stable */
    while (!(SYSCON_SYS_MODE_CTRL_XTAL_RDY_MASK & SYSCON->SYS_MODE_CTRL))
        ;
    SYSCON->CLK_CTRL = (SYSCON->CLK_CTRL & ~SYSCON_CLK_CTRL_SYS_CLK_SEL_MASK) | SYSCON_CLK_CTRL_SYS_CLK_SEL(1U);
    POWER_WritePmuCtrl1(SYSCON, SYSCON_PMU_CTRL1_OSC32M_DIS_MASK, SYSCON_PMU_CTRL1_OSC32M_DIS(1U));
}

void wakeup_by_capa_sensor(void)
{
    /* config PA15 as cs's input */
    IOCON_PinMuxSet(IOCON, 0U, 15U, IOCON_FUNC1);
    /* IOCON_PinMuxSet(IOCON, 0U, 14U, IOCON_FUNC1); */

    /* config PA15 to High-Z */
    SYSCON->PIO_PULL_CFG[0] &= ~0xC0000000U;

    CLOCK_EnableClock(kCLOCK_Cs);

    POWER_WritePmuCtrl1(SYSCON, SYSCON_PMU_CTRL1_CAP_SEN_DIS_MASK, SYSCON_PMU_CTRL1_CAP_SEN_DIS(0U));

    CS->CTRL0 =
        (CS->CTRL0 & ~(CS_CTRL0_CLK_DIV_MASK | CS_CTRL0_OSC_FREQ_MASK | CS_CTRL0_SRST_MASK | CS_CTRL0_ENABLE_MASK)) |
        (79U << CS_CTRL0_CLK_DIV_SHIFT) | (10U << CS_CTRL0_OSC_FREQ_SHIFT) | (0U << CS_CTRL0_CLK_DIV_SHIFT) |
        CS_CTRL0_ENABLE_MASK;

    CS->LP_CTRL =
        (CS->LP_CTRL &
         ~(CS_LP_CTRL_THR_MASK | CS_LP_CTRL_LP_CH_MASK | CS_LP_CTRL_LP_EN_MASK | CS_LP_CTRL_DEBONCE_NUM_MASK)) |
        (600U << CS_LP_CTRL_THR_SHIFT) | (1U << CS_LP_CTRL_LP_CH_SHIFT) | CS_LP_CTRL_LP_EN_MASK |
        (10U << CS_LP_CTRL_DEBONCE_NUM_SHIFT);

    /* after each scan loop, idle for some time. unit: 10ms */
    /* CS->IDLE_PERIOD = 50U; */

    CS->LP_INTEN = CS_LP_INTEN_LP_INTEN_MASK;
}

void CS_WAKEUP_IRQHandler(void)
{
    CS->LP_CTRL &= ~CS_LP_CTRL_LP_EN_MASK;
    NVIC_DisableIRQ(CS_WAKEUP_IRQn);
    NVIC_ClearPendingIRQ(CS_WAKEUP_IRQn);
}

/* #define CAP_SENSOR_WAKEUP */

/*!
 * @brief Main function
 */
int main(void)
{
    uint32_t msk, val;
    POWER_EnableDCDC(true);
    CALIB_SystemCalib();

    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    POWER_Init();

    POWER_EnablePD(kPDRUNCFG_PD_FIR);
    POWER_EnablePD(kPDRUNCFG_PD_FSP);
    POWER_EnablePD(kPDRUNCFG_PD_OSC32M);

    /* Enable OSC_EN as interrupt and wakeup source */
    //    SYSCON->PMU_CTRL0 = SYSCON->PMU_CTRL0 | SYSCON_PMU_CTRL0_OSC_INT_MSK_MASK;
    /* Power control 1 */
    msk = SYSCON_PMU_CTRL1_XTAL32K_PDM_DIS_MASK | SYSCON_PMU_CTRL1_RCO32K_PDM_DIS_MASK |
          SYSCON_PMU_CTRL1_XTAL32K_DIS_MASK | SYSCON_PMU_CTRL1_RCO32K_DIS_MASK;

#if (defined(BOARD_XTAL1_CLK_HZ) && (BOARD_XTAL1_CLK_HZ == 32768U))
    val = SYSCON_PMU_CTRL1_XTAL32K_PDM_DIS(0U)  /* switch on XTAL32K during power down */
          | SYSCON_PMU_CTRL1_RCO32K_PDM_DIS(1U) /* switch off RCO32K during power down */
          | SYSCON_PMU_CTRL1_XTAL32K_DIS(0U)    /* switch on XTAL32K */
          | SYSCON_PMU_CTRL1_RCO32K_DIS(1U);    /* switch off RCO32K at all time */
#else
    val = SYSCON_PMU_CTRL1_XTAL32K_PDM_DIS(1U)  /* switch off XTAL32K during power down */
          | SYSCON_PMU_CTRL1_RCO32K_PDM_DIS(0U) /* switch on RCO32K during power down */
          | SYSCON_PMU_CTRL1_XTAL32K_DIS(1U)    /* switch off XTAL32K at all time */
          | SYSCON_PMU_CTRL1_RCO32K_DIS(0U);    /* switch on RCO32K */
#endif

    /* The default setting of capacitive sensor, DAC, ADC and USB PLL's power are disabled.
       User should power on these peripherals when using them. */
    POWER_WritePmuCtrl1(SYSCON, msk, val);

    /* Enable GPIO clock */
    CLOCK_EnableClock(kCLOCK_Gpio);
    /* Use Low Frequency Clock source as wdt's clock source */
    CLOCK_AttachClk(k32K_to_WDT_CLK);
    /* Config BLE's high frequency clock to 8MHz */
    CLOCK_AttachClk(k8M_to_BLE_CLK);

    /* Disable ble core's clock. If ble core's clock is enabled, ahb can only be 8M, 16M or 32M */
    CLOCK_DisableClock(kCLOCK_Ble);

    //    BUTTON_Init();
    //    BUTTON_SetGpioWakeupLevel(BOARD_SW_GPIO, BOARD_SW1_GPIO_PIN, 0U);
    //    BUTTON_SetGpioWakeupLevel(BOARD_SW_GPIO, BOARD_SW2_GPIO_PIN, 0U);

    RTC_Init(RTC);

    /* Enable RTC free running interrupt */
    RTC_EnableInterrupts(RTC, kRTC_FreeRunningInterruptEnable);
    /* Enable at the NVIC */

    /* Enable gpio wakeup at button2 */
    SYSCON->PIO_WAKEUP_LVL0 = SYSCON->PIO_WAKEUP_LVL0 | BOARD_SW2_GPIO_PIN_MASK;
    SYSCON->PIO_WAKEUP_EN0 = SYSCON->PIO_WAKEUP_EN0 | BOARD_SW2_GPIO_PIN_MASK;


    while (1)
    {
        /* Wait for user input */

        __disable_irq();
        switch_to_OSC32M();
        POWER_LatchIO();
        CLOCK_DisableClock(kCLOCK_Flexcomm0);
        POWER_EnterPowerDown(0);
        CLOCK_EnableClock(kCLOCK_Flexcomm0);
        POWER_RestoreIO();
        switch_to_XTAL();
        /* after waking up from pwoer down, usart config is lost, recover it */
        BOARD_WakeupRestore();
        __enable_irq();
    }
}
